@@ -36,6 +36,26 @@ class GordoBaseDataset(metaclass=ABCMeta):
         Return X, y data as numpy or pandas' dataframes given current state
         """
 
+    def get_client_data(
+        self, build_dataset_metadata: dict
+    ) -> Tuple[
+        Union[np.ndarray, pd.DataFrame, xr.DataArray],
+        Union[np.ndarray, pd.DataFrame, xr.DataArray],
+    ]:
+        """
+        The version of `get_data` used by gordo-client
+
+        Parameters
+        ----------
+        build_dataset_metadata: dict
+            build_metadata.dataset part of the metadata
+
+        Returns
+        -------
+
+        """
+        return self.get_data()
+
     def to_dict(self) -> dict:
         """
         Serialize this object into a dict representation, which can be used to
