@@ -1,6 +1,8 @@
+import logging
+
 from abc import ABCMeta, abstractmethod
 
-from gordo_dataset.file_system import FileSystem
+from gordo_dataset.file_system import FileSystem, FileInfo
 from .file_type import FileType, ParquetFileType, CsvFileType, TimeSeriesColumns
 from .partition import Partition, YearPartition, MonthPartition
 
@@ -8,6 +10,9 @@ from typing import Iterable, Optional, List, Tuple, Type, Dict, cast
 
 from ..exceptions import ConfigException
 
+
+logger = logging.getLogger(__name__)
+
 time_series_columns = TimeSeriesColumns("Time", "Value", "Status")
 
 
@@ -31,7 +36,11 @@ class NcsFileType(metaclass=ABCMeta):
 
     @abstractmethod
     def paths(
-        self, fs: FileSystem, tag_name: str, partitions: Iterable[Partition]
+        self,
+        fs: FileSystem,
+        tag_dir: str,
+        tag_name: str,
+        partitions: Iterable[Partition],
     ) -> Iterable[Tuple[Partition, str]]:
         """
         Possible file paths for this file type. These paths should be relational to the tag directory
@@ -39,6 +48,7 @@ class NcsFileType(metaclass=ABCMeta):
         Parameters
         ----------
         fs: FileSystem
+        tag_dir: str
         tag_name: str
         partitions: Iterable[Partition]
 
@@ -68,18 +78,35 @@ class NcsMonthlyParquetFileType(NcsFileType):
         return MonthPartition
 
     def paths(
-        self, fs: FileSystem, tag_name: str, partitions: Iterable[Partition]
+        self,
+        fs: FileSystem,
+        tag_dir: str,
+        tag_name: str,
+        partitions: Iterable[Partition],
     ) -> Iterable[Tuple[Partition, str]]:
         file_extension = self._file_type.file_extension
+        month_partitions: List[MonthPartition] = []
         for partition in partitions:
             if not self.check_partition(partition):
                 raise NotImplementedError()
-            partition = cast(MonthPartition, partition)
-            file_name = (
-                f"{tag_name}_{partition.year}{partition.month:02d}{file_extension}"
-            )
-            path = fs.join("parquet", str(partition.year), file_name)
-            yield partition, path
+            month_partitions.append(cast(MonthPartition, partition))
+        tag_name_upper = tag_name.upper()
+        dirs_by_years: Dict[int, List[Tuple[str, Optional[FileInfo]]]] = {}
+        for partition in month_partitions:
+            if partition.year not in dirs_by_years:
+                dir_path = fs.join(tag_dir, "parquet", str(partition.year))
+                try:
+                    dirs_by_years[partition.year] = list(fs.ls(dir_path))
+                except FileNotFoundError:
+                    logger.info("File not found '%s'", dir_path)
+                    break
+            files_list = dirs_by_years[partition.year]
+            for path, file_info in files_list:
+                _, file_name = fs.split(path)
+                if cast(FileInfo, file_info).isfile():
+                    file_name_expected = f"{tag_name_upper}_{partition.year}{partition.month:02d}{file_extension.upper()}"
+                    if file_name.upper() == file_name_expected:
+                        yield partition, path
 
 
 class NcsYearlyParquetFileType(NcsFileType):
@@ -100,15 +127,34 @@ class NcsYearlyParquetFileType(NcsFileType):
         return YearPartition
 
     def paths(
-        self, fs: FileSystem, tag_name: str, partitions: Iterable[Partition]
+        self,
+        fs: FileSystem,
+        tag_dir: str,
+        tag_name: str,
+        partitions: Iterable[Partition],
     ) -> Iterable[Tuple[Partition, str]]:
         file_extension = self._file_type.file_extension
+        year_partitions: List[YearPartition] = []
         for partition in partitions:
             if not self.check_partition(partition):
                 raise NotImplementedError()
-            partition = cast(YearPartition, partition)
-            path = fs.join("parquet", f"{tag_name}_{partition.year}{file_extension}")
-            yield partition, path
+            year_partitions.append(cast(YearPartition, partition))
+        tag_name_upper = tag_name.upper()
+        for partition in year_partitions:
+            dir_path = fs.join(tag_dir, "parquet")
+            try:
+                files_list = list(fs.ls(dir_path))
+            except FileNotFoundError:
+                logger.info("File not found '%s'", dir_path)
+                break
+            for path, file_info in files_list:
+                _, file_name = fs.split(path)
+                if cast(FileInfo, file_info).isfile():
+                    file_name_expected = (
+                        f"{tag_name_upper}_{partition.year}{file_extension.upper()}"
+                    )
+                    if file_name.upper() == file_name_expected:
+                        yield partition, path
 
 
 class NcsCsvFileType(NcsFileType):
@@ -130,14 +176,19 @@ class NcsCsvFileType(NcsFileType):
         return self._partition_type
 
     def paths(
-        self, fs: FileSystem, tag_name: str, partitions: Iterable[Partition]
+        self,
+        fs: FileSystem,
+        tag_dir: str,
+        tag_name: str,
+        partitions: Iterable[Partition],
     ) -> Iterable[Tuple[Partition, str]]:
+        # TODO Make this case insensitive for tag_name
         file_extension = self._file_type.file_extension
         for partition in partitions:
             if not self.check_partition(partition):
                 raise NotImplementedError()
             path = f"{tag_name}_{partition.year}{file_extension}"
-            yield partition, path
+            yield partition, fs.join(tag_dir, path)
 
 
 ncs_file_types: Dict[str, Type[NcsFileType]] = {
