@@ -3,16 +3,16 @@ import logging
 from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
 import timeit
-from typing import Iterable, Iterator, List, Optional, Tuple, cast, Union
+from typing import Iterable, List, Optional, Tuple, cast, Union
 
 import pandas as pd
 
 from gordo_dataset.file_system.base import FileSystem
-from gordo_dataset.sensor_tag import SensorTag
+from gordo_dataset.sensor_tag import SensorTag, Tag
 from gordo_dataset.utils import capture_args
+from gordo_dataset.assets_config import AssetsConfig
 
 from .base import GordoBaseDataProvider
-from .assets_config import AssetsConfig
 from .ncs_contants import NCS_READER_NAME
 from .ncs_file_type import load_ncs_file_types, DEFAULT_TYPE_NAMES
 from .ncs_lookup import NcsLookup, TagLocations
@@ -89,8 +89,8 @@ class NcsReader(GordoBaseDataProvider):
             storage_name = storage.name
         self.storage_name: str = storage_name
         if ncs_lookup is None:
-            ncs_lookup = self.create_ncs_lookup(max_file_size)
-        elif isinstance(ncs_lookup, NcsLookup):
+            ncs_lookup = self.create_ncs_lookup(assets_config, max_file_size)
+        elif not isinstance(ncs_lookup, NcsLookup):
             raise ConfigException("ncs_lookup should be instance of NcsLookup")
         self.ncs_lookup = ncs_lookup
         self.partition_by = self.prepare_partition_by(partition_by)
@@ -107,10 +107,16 @@ class NcsReader(GordoBaseDataProvider):
             result = partition_by
         return cast(PartitionBy, result)
 
-    def create_ncs_lookup(self, max_file_size: Optional[int]) -> NcsLookup:
+    def create_ncs_lookup(
+        self, assets_config: AssetsConfig, max_file_size: Optional[int]
+    ) -> NcsLookup:
         ncs_file_types = load_ncs_file_types(self.lookup_for)
         return NcsLookup(
-            self.storage, ncs_file_types, self.storage_name, max_file_size=max_file_size
+            self.storage,
+            assets_config,
+            ncs_file_types,
+            self.storage_name,
+            max_file_size=max_file_size,
         )
 
     @property
@@ -120,11 +126,12 @@ class NcsReader(GordoBaseDataProvider):
         """
         return NCS_READER_NAME
 
-    def can_handle_tag(self, tag: SensorTag):
+    def can_handle_tag(self, tag: Union[str, SensorTag]):
         """
         Implements GordoBaseDataProvider, see base class for documentation
         """
-        return self.dl_base_path is not None or self.base_paths_from_asset(tag.asset)
+        # TODO do something better here
+        return True
 
     @staticmethod
     def filter_series(
@@ -140,10 +147,10 @@ class NcsReader(GordoBaseDataProvider):
         self,
         train_start_date: datetime,
         train_end_date: datetime,
-        tag_list: List[SensorTag],
+        tag_list: List[Union[str, SensorTag]],
         dry_run: Optional[bool] = False,
         **kwargs,
-    ) -> Iterable[pd.Series]:
+    ) -> Iterable[Tuple[pd.Series, Tag]]:
         """
         See GordoBaseDataProvider for documentation
         """
@@ -156,42 +163,46 @@ class NcsReader(GordoBaseDataProvider):
             split_by_partitions(self.partition_by, train_start_date, train_end_date)
         )
 
-        tag_dirs_iter = self.ncs_lookup.assets_config_tags_lookup(
-            self.assets_config, tag_list, self.dl_base_path
+        tag_lookups = self.ncs_lookup.assets_config_tags_lookup(
+            tag_list, self.dl_base_path
         )
 
         if self.threads:
             with ThreadPoolExecutor(max_workers=self.threads) as executor:
                 fetched_tags = executor.map(
-                    lambda tag_dirs: self._load_series_mapper(
-                        tag_dirs, partitions, dry_run
+                    lambda arg: self._load_series_mapper(
+                        arg[0], arg[1], partitions, dry_run
                     ),
-                    tag_dirs_iter,
+                    tag_lookups,
                 )
-                for series in fetched_tags:
-                    yield self.filter_series(series, train_start_date, train_end_date)
+                for series, tag in fetched_tags:
+                    yield self.filter_series(
+                        series, train_start_date, train_end_date
+                    ), tag
         else:
-            for tag_dirs in tag_dirs_iter:
-                series = self._load_series_mapper(tag_dirs, partitions, dry_run)
-                yield self.filter_series(series, train_start_date, train_end_date)
+            for tag_name, path_with_tag in tag_lookups:
+                series, tag = self._load_series_mapper(
+                    tag_name, path_with_tag, partitions, dry_run
+                )
+                yield self.filter_series(series, train_start_date, train_end_date), tag
 
     def _load_series_mapper(
         self,
-        tag_dirs: Tuple[SensorTag, Optional[str]],
+        tag_name: str,
+        path_with_tag: Optional[Tuple[str, SensorTag]],
         partitions: List[Partition],
         dry_run: Optional[bool] = False,
-    ) -> pd.Series:
-        tag, tag_dir = tag_dirs
-        if tag_dir is None:
+    ) -> Tuple[pd.Series, Tag]:
+        if path_with_tag is None:
             logger.info(
                 "Unable to find tag '%s' (asset '%s') directory in storage '%s'",
-                tag.name,
-                tag.asset,
+                tag_name,
                 self.storage_name,
             )
-            return pd.Series()
+            return pd.Series(), tag_name
+        tag_dir, tag = path_with_tag
         tag_locations = self.ncs_lookup.files_lookup(tag_dir, tag, partitions)
-        return self.read_tag_locations(tag_locations, dry_run)
+        return self.read_tag_locations(tag_locations, dry_run), tag
 
     def read_tag_locations(
         self, tag_locations: TagLocations, dry_run: Optional[bool] = False
@@ -266,80 +277,3 @@ class NcsReader(GordoBaseDataProvider):
             Path of tag to be checked if exists.
         """
         fs.info(path)
-
-    def read_tag_files(
-        self,
-        tag: SensorTag,
-        partitions: Iterable[Partition],
-        dry_run: Optional[bool] = False,
-    ) -> pd.Series:
-        """
-        Download tag files for the given partitions into dataframes,
-        and return as one dataframe.
-
-        Parameters
-        ----------
-        tag: SensorTag
-            the tag to download data for
-        partitions: Iterable[Partition]
-            range object providing partitions to include
-        dry_run: Optional[bool]
-            if True, don't download data, just check info, log, and return
-
-        Returns
-        -------
-        pd.Series:
-            Series with all partitions for one tag.
-        """
-        tag_base_paths: List[str] = []
-        if self.dl_base_path:
-            tag_base_paths = [self.dl_base_path]
-        if not tag_base_paths:
-            base_paths = self.base_paths_from_asset(tag.asset)
-            if base_paths:
-                tag_base_paths = base_paths
-
-        if not tag_base_paths:
-            raise ValueError(f"Unable to find base path from tag {tag} ")
-        logger.info(f"Downloading tag: {tag} for partitions: {partitions}")
-
-        tag_dir = None
-        for found_tag, dir_path in self.ncs_lookup.tag_dirs_lookup(
-            tag_base_paths, [tag]
-        ):
-            if found_tag == tag:
-                tag_dir = dir_path
-                break
-        if tag_dir:
-            raise FileNotFoundError(
-                f"Unable to find location of {tag} in storage {self.storage_name}"
-            )
-
-        tag_locations = self.ncs_lookup.files_lookup(
-            cast(str, tag_dir), tag, partitions
-        )
-        return self.read_tag_locations(tag_locations, dry_run)
-
-    def base_paths_from_asset(self, asset: str) -> Optional[List[str]]:
-        """
-        Resolves an asset code to the datalake basepath containing the data.
-        Returns None if it does not match any of the asset codes we know.
-        """
-        if not asset:
-            return None
-
-        logger.debug(f"Looking for match for asset {asset}")
-        asset = asset.lower()
-        assets_config = self.assets_config
-        path_spec = assets_config.get_path(self.storage_name, asset)
-        if path_spec is None:
-            return None
-        if path_spec.reader != self.reader_name:
-            raise ValueError(
-                "Assets reader name should be equal '%s' and not '%s'"
-                % (self.reader_name, path_spec.reader)
-            )
-        full_paths = path_spec.full_paths(self.storage)
-
-        logger.debug(f"Found asset code {asset}, returning {full_paths}")
-        return full_paths
