@@ -11,7 +11,7 @@ from concurrent.futures import ThreadPoolExecutor
 
 from gordo_dataset.file_system import FileSystem
 from gordo_dataset.utils import capture_args
-from gordo_dataset.sensor_tag import SensorTag
+from gordo_dataset.sensor_tag import SensorTag, unique_tag_names, Tag
 
 from gordo_dataset.data_provider.base import GordoBaseDataProvider
 from gordo_dataset.data_provider.storages import create_storage, DEFAULT_STORAGE_TYPE
@@ -164,20 +164,27 @@ class NesDataProvider(GordoBaseDataProvider):
         self,
         train_start_date: datetime,
         train_end_date: datetime,
-        tag_list: List[SensorTag],
+        tag_list: List[Union[str, SensorTag]],
         dry_run: Optional[bool] = False,
         **kwargs,
-    ) -> Iterable[pd.Series]:
+    ) -> Iterable[Tuple[pd.Series, Tag]]:
+        for tag in tag_list:
+            if type(tag) is str:
+                raise ConfigException(
+                    "'%s' tag does not have an asset provided" % cast(str, tag)
+                )
+        sensor_tag_list = cast(List[SensorTag], tag_list)
         if train_end_date < train_start_date:
             raise ConfigException(
                 f"NCS reader called with train_end_date: {train_end_date} before train_start_date: {train_start_date}"
             )
-        measurements = self.measurement_mapper.get_measurements(tag_list)
+        measurements = self.measurement_mapper.get_measurements(sensor_tag_list)
         partitions = split_by_partitions(
             PartitionBy.MONTH, train_start_date, train_end_date
         )
         file_locations = self.lookup.files_lookup(measurements, partitions)
-        series: Dict[str, pd.Series] = dict()
+        series: Dict[str, pd.Series] = {}
+        tag_names = unique_tag_names(tag_list)
         for file_location, s in self.load_files(file_locations, bool(dry_run)):
             name = s.name
             if name not in series:
@@ -192,6 +199,7 @@ class NesDataProvider(GordoBaseDataProvider):
                         e,
                     )
         for s in series.values():
+            tag = tag_names[s.name]
             filtered = s[(s.index >= train_start_date) & (s.index < train_end_date)]
             filtered.sort_index(inplace=True)
-            yield filtered
+            yield filtered, tag
