@@ -1,6 +1,6 @@
 from urllib.parse import quote
 
-from typing import Iterable, Tuple
+from typing import Iterable, Tuple, Union
 from gordo_dataset.data_provider.storages import FileSystem
 
 
