@@ -11,14 +11,14 @@ from azure.storage.filedatalake import (
 )
 from azure.identity import (
     ClientSecretCredential,
+    AzureCliCredential,
     InteractiveBrowserCredential,
     DefaultAzureCredential,
+    ChainedTokenCredential,
 )
 from typing import Optional, Any, IO, Iterable, cast, Tuple
 from io import BytesIO, TextIOWrapper
 
-from gordo_dataset.exceptions import ConfigException
-
 from .base import FileSystem, FileInfo, FileType
 from .azure import ADLSecret
 
@@ -69,7 +69,9 @@ class ADLGen2FileSystem(FileSystem):
         """
         if interactive:
             logger.info("Attempting to use interactive azure authentication")
-            credential = InteractiveBrowserCredential()
+            credential = ChainedTokenCredential(
+                AzureCliCredential(), InteractiveBrowserCredential()
+            )
         elif type(adl_secret) is ADLSecret:
             adl_secret = cast(ADLSecret, adl_secret)
             logger.info("Attempting to use datalake service authentication")
@@ -243,6 +245,8 @@ class ADLGen2FileSystem(FileSystem):
                     self.path_properties_to_info(properties) if with_info else None
                 )
                 yield properties.name, file_info
+        except ResourceNotFoundError:
+            raise FileNotFoundError(path)
         except AttributeError as e:
             self._handle_attribute_error_bug(e)
 
@@ -257,5 +261,7 @@ class ADLGen2FileSystem(FileSystem):
                     self.path_properties_to_info(properties) if with_info else None
                 )
                 yield properties.name, file_info
+        except ResourceNotFoundError:
+            raise FileNotFoundError(base_path)
         except AttributeError as e:
             self._handle_attribute_error_bug(e)
